/** @file config.h

	Skelton for retropc emulator

	@author Takeda.Toshiya
	@date   2006.08.18 -

	@note
	Modified for MBS1 by Sasaji at 2015.08.16

	@brief [ config ]
	
	@note
	Config ini file uses UTF-8 as string code.
	Under windows: convert to MBCS(shift jis) code from UTF-8.
	Under mac/linux: always uses UTF-8, so LANG environment should be set UTF-8. 
*/

#ifndef _CONFIG_H_
#define _CONFIG_H_

#include "common.h"
#include "vm/vm.h"
#include "cchar.h"
#include "cptrlist.h"
#ifdef _UNICODE
#define SI_CONVERT_GENERIC	// use generic simple.ini on all platform
#endif
#include "SimpleIni.h"

#define FILE_VERSION	0x30

#define IOPORT_MSK_5FDD		0x001
#define IOPORT_MSK_3FDD		0x002
#define IOPORT_MSK_PSG6		0x004
#define IOPORT_MSK_EXPIA	0x008
#define IOPORT_MSK_EXACIA	0x010
#define IOPORT_MSK_PSG9		0x020
#define IOPORT_MSK_KANJI	0x040
#define IOPORT_MSK_EXPSG	0x080
#define IOPORT_MSK_FDDALL	0x003
#define IOPORT_MSK_OS9BD	0x100
#define IOPORT_MSK_CM01		0x200
#define IOPORT_MSK_KEYBD	0x400
#define IOPORT_MSK_MOUSE	0x800
#define IOPORT_MSK_ALL		0xfff
#define IOPORT_NUMS			12
#define IOPORT_USE_5FDD		(config.io_port & IOPORT_MSK_5FDD)
#define IOPORT_USE_3FDD		(config.io_port & IOPORT_MSK_3FDD)
#define IOPORT_USE_PSG6		(config.io_port & IOPORT_MSK_PSG6)
#define IOPORT_USE_EXPIA	(config.io_port & IOPORT_MSK_EXPIA)
#define IOPORT_USE_EXACIA	(config.io_port & IOPORT_MSK_EXACIA)
#define IOPORT_USE_PSG9		(config.io_port & IOPORT_MSK_PSG9)
#define IOPORT_USE_KANJI	(config.io_port & IOPORT_MSK_KANJI)
#define IOPORT_USE_EXPSG	(config.io_port & IOPORT_MSK_EXPSG)
#define IOPORT_USE_FDD		(config.io_port & IOPORT_MSK_FDDALL)
#define IOPORT_USE_OS9BD	(config.io_port & IOPORT_MSK_OS9BD)
#define IOPORT_USE_CM01		(config.io_port & IOPORT_MSK_CM01)
#define IOPORT_USE_KEYBD	(config.io_port & IOPORT_MSK_KEYBD)
#define IOPORT_USE_MOUSE	(config.io_port & IOPORT_MSK_MOUSE)

#define MSK_SHOWLEDBOX		0x01
#define MSK_SHOWMSGBOARD	0x02
#define MSK_USEJOYSTICK		0x04
#define MSK_INSIDELEDBOX	0x08
#define MSK_LEDBOX_ALL		0x09
#define MSK_USELIGHTPEN		0x10
#define MSK_USEMOUSE		0x20
#define MSK_USEPIAJOYSTICK	0x40
#define MSK_USEJOYSTICK_ALL	0x44
#define MSK_SHOWDLG_ALL		0xff
#define FLG_SHOWLEDBOX		(config.show_dlg & MSK_SHOWLEDBOX)
#define FLG_SHOWMSGBOARD	(config.show_dlg & MSK_SHOWMSGBOARD)
#define FLG_USEJOYSTICK		(config.show_dlg & MSK_USEJOYSTICK)
#define FLG_INSIDELEDBOX	(config.show_dlg & MSK_INSIDELEDBOX)
#define FLG_LEDBOX_ALL		(config.show_dlg & MSK_LEDBOX_ALL)
#define FLG_USELIGHTPEN		(config.show_dlg & MSK_USELIGHTPEN)
#define FLG_USEMOUSE		(config.show_dlg & MSK_USEMOUSE)
#define FLG_USEPIAJOYSTICK	(config.show_dlg & MSK_USEPIAJOYSTICK)
#define FLG_USEJOYSTICK_ALL	(config.show_dlg & MSK_USEJOYSTICK_ALL)

#define MSK_ORIG_CURIG		0x01
#define MSK_ORIG_PAL8		0x02
#define MSK_ORIG_NOPAL		0x04
#define MSK_ORIG_MOUSEEG	0x08
#define MSK_ORIG_LIMKEY		0x10
#define MSK_ORIG_FDMSK		0x20
#define MSK_ORIG_FDINSERT	0x40
#define MSK_ORIG_FDDRQ		0x80
#define FLG_ORIG_CURIG		(config.original & MSK_ORIG_CURIG)
#define FLG_ORIG_PAL8		(config.original & MSK_ORIG_PAL8)
#define FLG_ORIG_NOPAL		(config.original & MSK_ORIG_NOPAL)
#define FLG_ORIG_MOUSEEG	(config.original & MSK_ORIG_MOUSEEG)
#define FLG_ORIG_LIMKEY		(config.original & MSK_ORIG_LIMKEY)
#define FLG_ORIG_FDMSK		(config.original & MSK_ORIG_FDMSK)
#define FLG_ORIG_FDINSERT	(config.original & MSK_ORIG_FDINSERT)
#define FLG_ORIG_FDDRQ		(config.original & MSK_ORIG_FDDRQ)


/// directory path
class CDirPath : public CTchar
{
public:
	void SetFromPath(const _TCHAR *file_path);
	void Set(const _TCHAR *dir_path);
private:
	void Set(const CDirPath &src) {}
	void Set(const _TCHAR *src_str, int src_len) {}
};

/// file path
class CFilePath : public CTchar
{
};

#define MAX_HISTORY	10

/// recent path
class CRecentPath
{
public:
	CTchar path;
	int    num;
public:
	CRecentPath();
	CRecentPath(const CRecentPath &src);
	CRecentPath(const _TCHAR *srcpath, int srcnum);
	~CRecentPath();
	void Set(const _TCHAR *srcpath, int srcnum);
	void Clear();
	bool Match(const _TCHAR *tagpath, int tagnum);
};

/// recent path list
class CRecentPathList : public CPtrList<CRecentPath>
{
public:
	bool updated;
public:
	CRecentPathList();
	~CRecentPathList();
	bool Match(const _TCHAR *tagpath, int tagnum);
	void Update(const _TCHAR *tagpath, int tagnum);
};

/// Read/Write config file
///
///	@note
///	Config ini file uses UTF-8 as string code.
///	Under windows: convert to MBCS(shift jis) code from UTF-8.
///	Under mac/linux: always uses UTF-8, so LANG environment should be set UTF-8. 
class Config
{
public:
	int version1;	// config file version
	int version2;

	// recent files
#ifdef USE_CART
	CDirPath initial_cart_path;
	CRecentPath recent_cart_path;
#endif
#ifdef USE_FD1
	CDirPath initial_disk_path;
#if defined(USE_FD8) || defined(USE_FD7)
	CRecentPathList recent_disk_path[8];
	CRecentPath opened_disk_path[8];
#elif defined(USE_FD6) || defined(USE_FD5)
	CRecentPathList recent_disk_path[6];
	CRecentPath opened_disk_path[6];
#else
	CRecentPathList recent_disk_path[4];
	CRecentPath opened_disk_path[4];
#endif
#endif
#ifdef USE_QUICKDISK
	CDirPath initial_quickdisk_path;
	CRecentPath recent_quickdisk_path;
#endif
#ifdef USE_DATAREC
	CDirPath initial_datarec_path;
	CRecentPathList recent_datarec_path;
	CRecentPath opened_datarec_path;
#endif
#ifdef USE_MEDIA
	CDirPath initial_media_path;
	CRecentPath recent_media_path;
#endif
#ifdef USE_BINARY_FILE1
	CDirPath initial_binary_path;
	CRecentPath recent_binary_path[2];
#endif

	// screen
	int window_mode;
	uint8 stretch_screen;
//	bool cutout_screen;
	int pixel_aspect;
	uint8 capture_type;

	// sound
	int sound_frequency;
	int sound_latency;

	// virtual machine
	int cpu_power;
#ifdef USE_FD1
	bool ignore_crc;
#endif
#ifdef USE_DIPSWITCH
	uint8 dipswitch;
#endif
#ifdef USE_BOOT_MODE
	int boot_mode;		// MZ-800, PASOPIA, PC-8801MA, PC-98DO
#endif
#ifdef USE_CPU_CLOCK_LOW
	bool cpu_clock_low;	// PC-8801MA, PC-9801E, PC-9801VM, PC-98DO
#endif
#if defined(_HC80) || defined(_PASOPIA) || defined(_PC8801MA)
	int device_type;
#endif
#if defined(USE_MONITOR_TYPE) || defined(USE_SCREEN_ROTATE)
	int monitor_type;
#endif
#ifdef _MBS1
	uint8 sys_mode;
	bool sync_irq;
#endif
#ifdef USE_SCANLINE
	uint8 scan_line;
#endif
#ifdef _MBS1
	int  afterimage;
	uint8 exram_size_num;
	bool mem_nowait;
	uint8 tvsuper;
	uint8 use_opengl;
	int8  disptmg_skew;
	int8  curdisp_skew;
	uint8 gl_filter_type;
	int volume;
	int fdd_type;
	int mount_fdd;

	// bit0: use fdd5  bit1: use fdd3  bit2: use psg  bit3: use ex pia
	// bit4: use ex acia  bit5: use 9psg  bit6: use kanji
	// bit7: use s1 ex psg (CONFIG_VERSION >= 0x02)
	int io_port;
	// bit0: show led  bit1: show msg  bit2: use joystick  bit3: inside led
	// bit4: enable lightpen bit5: enable mouse bit6: use pia joystick
	uint8 show_dlg;
	// bit0: show cursor on IG  bit1: set dark gray on palette 8  bit2: not use palette in ANALOG mode
	// bit3: occur mouse IRQ when both edge
	// bit4: auto key release after few frames
	// bit5: ff00 mask  bit6: check fd insert when get track  bit7: ignore drq when sector is not found
	int original;

	CDirPath initial_printer_path;
	CDirPath initial_state_path;
	CRecentPath saved_state_path;

	int fps_no;

	int disp_device_no;
	int screen_width;
	int screen_height;

	CDirPath initial_autokey_path;
	CRecentPath opened_autokey_path;
	// support above params on cfg file
	CTchar msgboard_info_fontname;
	CTchar msgboard_msg_fontname;
	CTchar menu_fontname;
	CDirPath rom_path;

	bool realmode_datarec;

	//
	uint8 screen_video_size;
	uint8 msgboard_info_fontsize;
	uint8 msgboard_msg_fontsize;
	uint8 menu_fontsize;

	int beep_volume;
	int psg_volume;
	int psg6_volume;
	int psg9_volume;
	int relay_volume;
	int cmt_volume;
	int fdd_volume;
	bool mute;
	bool beep_mute;
	bool psg_mute;
	bool psg6_mute;
	bool psg9_mute;
	bool relay_mute;
	bool cmt_mute;
	bool fdd_mute;

	CTchar printer_server_host[MAX_PRINTER];
	int    printer_server_port[MAX_PRINTER];
	bool   printer_direct[MAX_PRINTER];

	int    comm_dipswitch[MAX_COMM];
	CTchar comm_server_host[MAX_COMM];
	int    comm_server_port[MAX_COMM];
	bool   comm_server[MAX_COMM];
	bool   comm_connect[MAX_COMM];
	bool   comm_through[MAX_COMM];
	bool   comm_binary[MAX_COMM];

	CDirPath snapshot_path;

	uint8 led_pos;
	VmPoint led_dist[2];

	CDirPath font_path;

	bool reckey_recording;
	bool reckey_playing;

	bool wav_reverse;
	bool wav_half;
	bool wav_correct;
	uint8 wav_correct_type;

	uint8 wav_sample_rate;
	uint8 wav_sample_bits;

#endif
#ifdef USE_SOUND_DEVICE_TYPE
	int sound_device_type;
#endif

#ifdef USE_DEBUGGER
	int    debugger_imm_start;
	CTchar debugger_server_host;
	int    debugger_server_port;
#endif

public:
	Config();
	~Config();

	void initialize();
	void load(const _TCHAR *path);
	void save();
	void release();

	static bool get_number_in_path(_TCHAR *path, int *number);
	static bool set_number_in_path(_TCHAR *path, int number);

private:
	CSimpleIni *ini;

	CTchar ini_file;

	bool load_ini_file(const _TCHAR *file);
	void save_ini_file(const _TCHAR *file);

	const _TCHAR *conv_to_npath(const _TCHAR *path);
	const _TCHAR *conv_from_npath(const _TCHAR *npath);
	void get_str_value(const _TCHAR *section, const _TCHAR *key, CTchar &str);
	void get_dirpath_value(const _TCHAR *section, const _TCHAR *key, CDirPath &path);
	void get_filepath_value(const _TCHAR *section, const _TCHAR *key, CFilePath &path);
	void get_recentpath_value(const _TCHAR *section, const _TCHAR *key, CRecentPathList &pathlist);
};

extern Config *pconfig;
#define config (*pconfig)

#endif

