/** @file depend.h

	Skelton for retropc emulator


	@author Sasaji
	@date   2012.03.01

	@brief [ dependency definition ]
*/

#ifndef _DEPEND_H_
#define _DEPEND_H_

// disable warnings C4189, C4995 and C4996 for microsoft visual c++ 2005
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
#pragma warning( disable : 4819 )
#pragma warning( disable : 4995 )
#pragma warning( disable : 4996 )
#endif

// check memory leaks
#if defined(_DEBUG) && defined(_WIN32) && !defined(__MINGW32__)
//#define _CRTDBG_MAP_ALLOC
#undef _malloca
#include <stdlib.h>
#include <crtdbg.h>
#define malloc(s) _malloc_dbg(s, _NORMAL_BLOCK, __FILE__, __LINE__)
#if _MSC_VER > 1500
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif
#endif

#include <SDL.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef _WIN32
/* windows */
#include <windows.h>
#include <tchar.h>

# ifdef __MINGW32__
#  include <getopt.h>
# endif

#define USE_REC_VIDEO
#define USE_REC_VIDEO_VFW
#define USE_REC_VIDEO_FFMPEG	1
#define USE_REC_VIDEO_WIN		1

#define USE_REC_AUDIO
#define USE_REC_AUDIO_WAVE
#define USE_REC_AUDIO_FFMPEG	2

#elif (defined(__APPLE__) && defined(__MACH__))
/* macosx */
#include <unistd.h>
#include "extra/tchar.h"
#define __stdcall

#define USE_REC_VIDEO
#define USE_REC_VIDEO_AVKIT		0x01
#define USE_REC_VIDEO_QTKIT		0x02
#define USE_REC_VIDEO_FFMPEG	1
#define USE_REC_VIDEO_COCOA		1

#define USE_REC_AUDIO
#define USE_REC_AUDIO_WAVE
#define USE_REC_AUDIO_AVKIT		0x10
#define USE_REC_AUDIO_FFMPEG	2

#else
/* linux (Unix) */
#include <unistd.h>
#include "extra/tchar.h"
#define __stdcall

#define USE_LIB_GTK
//#define USE_X11_LEDBOX

#define USE_REC_VIDEO
#define USE_REC_VIDEO_FFMPEG	1
#define USE_REC_VIDEO_LIBPNG	1

#define USE_REC_AUDIO
#define USE_REC_AUDIO_WAVE
#define USE_REC_AUDIO_FFMPEG	2

#endif

#ifndef _MAX_PATH
# define _MAX_PATH	(260)
#endif

/* set compatible for SDL1 */
#ifndef USE_SDL2
/* SDL1 */

# define SDL_Keycode		SDLKey

# define SDL_WINDOW_FULLSCREEN	SDL_FULLSCREEN
# define SDL_WINDOW_OPENGL		SDL_OPENGL

#else
/* SDL2 */

# if defined(GUI_TYPE_GTK_X11)
//#  define GUI_USE_FOREIGN_WINDOW
# endif

#endif /* USE_SDL2 */

/* Support Outside Ledbox */
#if defined(_WIN32) \
 ||(defined(__APPLE__) && defined(__MACH__)) \
 || defined(USE_SDL2_LEDBOX) \
 || defined(USE_X11_LEDBOX)
#define USE_OUTSIDE_LEDBOX 1
#endif


#ifdef _UNICODE
	/* _UNICODE */

# ifndef _WIN32
wchar_t *_wgetenv(const wchar_t *);
int _wsystem(const wchar_t *);
int swprintf(wchar_t *ws, const wchar_t *format, ...);
int vswprintf(wchar_t *ws, const wchar_t *format, va_list arg);

#  ifdef _tcsicmp
#  undef _tcsicmp
#  endif
#  define _tcsicmp wcscasecmp

# endif /* !_WIN32 */

#else	/* !_UNICODE */

# ifndef _WIN32

#  ifdef _tcsicmp
#  undef _tcsicmp
#  endif
#  define _tcsicmp strcasecmp

# endif /* !_WIN32 */

#endif	/* _UNICODE */

#endif	/* _DEPEND_H_ */
