/** @file ledbox.h

	HITACHI MB-S1 Emulator 'EmuB-S1'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.03.08 -

	@brief [ led box ]
*/

#ifndef _LEDBOX_H_
#define _LEDBOX_H_

#include "../vm/vm.h"
#include <SDL.h>
#include "../common.h"

/**
	@brief LedBox is the class that display the access indicator on the screen.
*/
class LedBoxBase
{
protected:
	SDL_Surface *surface;
	uint64		flag_old;

	enum enum_led_type {
		LED_TYPE_VR = 0,
		LED_TYPE_VG,
		LED_TYPE_HR,
		LED_TYPE_HG,
		LED_TYPE_RR,
		LED_TYPE_CL,
		LCD_TYPE_TAPE,
		LCD_TYPE_FDD0,
		LCD_TYPE_FDD1,
		LCD_TYPE_FDD2,
		LCD_TYPE_FDD3,
		LCD_TYPE_3_FDD35H,
		LCD_TYPE_5_FDD35H,
		LCD_TYPE_H_FDD35H,
		LCD_TYPE_AB_A,
		LCD_TYPE_AB_B,
		LCD_TYPE_12M_1,
		LCD_TYPE_12M_2,
		LCD_TYPE_SU_S,
		LCD_TYPE_SU_U,
		LED_TYPE_DIGIT,
		LED_TYPE_PARTS,
		LED_TYPE_BASE,
		LED_TYPE_END
	};

	SDL_Surface *led[LED_TYPE_END];

	enum enum_led_pos {
		LED_POS_BASE = 0,
		LED_POS_KATAKANA,
		LED_POS_HIRAGANA,
		LED_POS_CAPS,
		LED_POS_PLAY,
		LED_POS_REC,
		LCD_POS_PLAY,
		LCD_POS_REC,
		LED_POS_FDD0,
		LED_POS_FDD1,
		LED_POS_FDD2,
		LED_POS_FDD3,
		LCD_POS_FDD0,
		LCD_POS_FDD1,
		LCD_POS_FDD2,
		LCD_POS_FDD3,
		LCD_POS_FDD35H,
		LCD_POS_AB,
		LCD_POS_12M,
		LCD_POS_SU,
		LED_POS_DIGIT1,
		LED_POS_DIGIT2,
		LED_POS_DIGIT3,
		LED_POS_END
	};
	// LEDʒu
	SDL_Rect led_pt[LED_POS_END];


	// \ʒu
	struct win_pt_st {
		int left;
		int top;
		int right;
		int bottom;
		int place;
	} win_pt;
	SDL_Rect    parent_pt;

	int			mode;
	int         prev_mode;
	VmPoint		dist_set[2];
	VmPoint		dist;

	bool        enable;
	bool        visible;
	bool        inside;

	bool create_surface(SDL_PixelFormat *);
	bool create_bitmap(const char *, const char *, SDL_Surface **);
	bool create_parts(SDL_Surface *srcsuf, int x, int y, int w, int h, SDL_Surface **suf);
	void copy_led(SDL_Rect &, SDL_Surface *);
	void copy_led_s(SDL_Rect &p, SDL_Surface *l, SDL_Rect &re);

	/* for dialog box */
	virtual void create_dialog_box_sub() {}
	virtual void show_dialog() {}
	virtual void move_in_place(int place) {}
	virtual void need_update_dialog() {}
public:
	LedBoxBase();
	virtual ~LedBoxBase();

	bool InitScreen(const char *res_path, SDL_PixelFormat *pixel_format);
	void Show(int);
	void SetMode(int);
	void SetPos(int left, int top, int right, int bottom, int place);
	void SetPos(int place);
	void Draw(SDL_Surface *);
	void Update(uint64 flag);

	void SetDistance(int place, const VmPoint *ndist);
	void GetDistance(VmPoint *ndist);

	bool IsEnable() { return enable; }

	/* for dialog box */
	virtual void CreateDialogBox() {}
	virtual void Move() {}
};

#if defined(USE_SDL2) && defined(USE_SDL2_LEDBOX)
#include "sdl2_ledbox.h"
#elif defined(_WIN32)
#include "windows/win_ledbox.h"
#elif defined(__MACH__) && defined(__APPLE__)
#include "cocoa/cocoa_ledbox.h"
#elif defined(SDL_VIDEO_DRIVER_X11) && defined(USE_X11_LEDBOX)
#include "gtk_x11/x11_ledbox.h"
#else
class LedBox : public LedBoxBase
{
public:
	LedBox() : LedBoxBase() {};
};
#endif

#endif /* _LEDBOX_H_ */
