/** @file win_gui.h

	HITACHI MB-S1 Emulator 'EmuB-S1'
	Skelton for retropc emulator

	@author Sasaji
	@date   2014.03.24 -

	@brief [ gui for windows ]
*/

#ifndef _WIN_GUI_H_
#define _WIN_GUI_H_

#include <Windows.h>
#include "../gui_base.h"
#include "../../common.h"
#include "winfont.h"
#include "win_configbox.h"
#include "win_keybindbox.h"
#include "win_volumebox.h"
#include "win_recvidbox.h"
#include "win_recaudbox.h"
#include "win_aboutbox.h"

namespace GUI_WIN
{

bool start_auto_key_from_clipboard(EMU *emu);

#ifdef GUI_TYPE_WINDOWS

class GUI : public GUI_BASE
{
private:
	HWND hWindow;
	HINSTANCE hInstance;
	CFont *font;
	ConfigBox *configbox;
	KeybindBox *keybindbox;
	VolumeBox *volumebox;
	RecVideoBox *recvidbox;
	RecAudioBox *recaudbox;
	AboutBox *aboutbox;

	/// menu
	bool now_menu;
	bool now_menuloop;

	static HMENU get_sub_menu(HMENU hMenu, int id);
	void update_recent_menu(HMENU hMenu, int menu_id, CRecentPathList &list);

	WNDPROC OrigProcPtr;
	/// Hook event procedure on main window and process message before on SDL.
	static LRESULT CALLBACK MessageProcStatic(HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam);
	LRESULT MessageProc(HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam);

public:
	GUI(int argc, char **argv, EMU *new_emu);
	virtual ~GUI();

	virtual int CreateWidget(SDL_Surface *screen, int width, int height);

	virtual int CreateMenu();

	virtual void ShowMenu();
	virtual void HideMenu();

	virtual void PreProcessEvent();
	virtual void PostProcessEvent();

	virtual void ScreenModeChanged(bool fullscreen);
	virtual bool NeedUpdateScreen();

	void SetWindowHandle(HWND new_hwnd) { hWindow = new_hwnd; }

	virtual void SetFocusToMainWindow();

	virtual bool ShowRecordVideoDialog(int fps_num);
	virtual bool ShowRecordAudioDialog(void);
	virtual bool ShowRecordVideoAndAudioDialog(int fps_num);

	virtual bool ShowAboutDialog();
#ifdef USE_EMU_INHERENT_SPEC
	virtual bool ShowConfigureDialog();
	virtual bool ShowKeybindDialog();
	virtual bool ShowVolumeDialog();
#endif
#ifdef USE_DATAREC
	virtual bool ShowLoadDataRecDialog();
	virtual bool ShowSaveDataRecDialog();
	virtual void set_datarec_file_menu(HMENU hMenu, UINT uItem);
#endif

#ifdef USE_FD1
	virtual bool ShowOpenFloppyDiskDialog(int drv);
	virtual int  ShowSelectFloppyDriveDialog(int drv);
	void set_disk_file_menu(HMENU hMenu, UINT uItem, int drv);
#ifdef USE_EMU_INHERENT_SPEC
	void set_disk_side_menu(HMENU hMenu, UINT uItem, int drv);
#endif
#endif

#ifdef USE_AUTO_KEY
	virtual bool ShowOpenAutoKeyDialog();
	virtual bool StartAutoKey(void);
#endif

#ifdef USE_EMU_INHERENT_SPEC
	virtual bool ShowSaveStateDialog(bool cont);
	virtual bool ShowLoadStateDialog();
#ifdef USE_KEY_RECORD
	virtual bool ShowPlayRecKeyDialog();
	virtual bool ShowRecordRecKeyDialog();
	virtual bool ShowRecordStateAndRecKeyDialog();
#endif
#endif

#ifdef USE_PRINTER
	virtual bool ShowSavePrinterDialog(int drv);
#endif

#ifdef USE_CART
	void open_cart_dialog();
#endif
#ifdef USE_QUICKDISK
	void open_quickdisk_dialog();
#endif
#ifdef USE_MEDIA
	void open_media_dialog();
#endif
#ifdef USE_BINARY_FILE1
	void open_binary_dialog(int drv, bool load);
#endif

#ifndef USE_SDL2
	virtual bool OpenDroppedFile(void *param);
#endif

	void update_menu(HMENU hMenu, int pos);
	void translate_menu(HMENU hMenu, int depth);

	static const char *gettext_acp(const char *src);
};

#endif /* GUI_TYPE_WINDOWS */

}; /* namespace GUI_WIN */

#if defined(GUI_TYPE_WINDOWS) && !defined(NOT_USE_GETTEXT_ACP)
#ifdef USE_GETTEXT
#ifdef _tgettext
#undef _
#undef _tgettext
#endif
#define _(text) GUI_WIN::GUI::gettext_acp(text)
#define _tgettext GUI_WIN::GUI::gettext_acp
#endif
#endif

#endif /* _WIN_GUI_H_ */
