/** @file msgboard.cpp

	HITACHI MB-S1 Emulator 'EmuB-S1'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.06.26 -

	@brief [ message board ]
*/
#include "msgboard.h"
#include "emu.h"
#include "config.h"

#undef USE_BG_TRANSPARENT

#define RECT_IN(st, xx, yy, ww, hh) { \
	st.x = xx; \
	st.y = yy; \
	st.w = ww; \
	st.h = hh; \
}

#define RGB_IN(st, rr, gg, bb) { \
	st.r = rr; \
	st.g = gg; \
	st.b = bb; \
}

MsgBoard::MsgBoard(EMU *pEmu)
{
	visible = true;
	enable  = false;

	sMainSuf = NULL;
	emu = pEmu;

	msg.font = NULL;
	info.font = NULL;

	inited = false;
}

MsgBoard::~MsgBoard()
{
	if (inited) {
		if (msg.font != NULL) TTF_CloseFont(msg.font);
		if (info.font != NULL) TTF_CloseFont(info.font);
		TTF_Quit();
	}
	if (sMainSuf != NULL) SDL_FreeSurface(sMainSuf);
}

void MsgBoard::CreateSurface(SDL_PixelFormat *format, int width, int height)
{
	if (sMainSuf != NULL) {
		SDL_FreeSurface(sMainSuf);
	}
	sMainSuf = SDL_CreateRGBSurface(SDL_SWSURFACE, width, height, format->BitsPerPixel,
			format->Rmask, format->Gmask, format->Bmask, format->Amask);
	if (sMainSuf == NULL) {
		enable = false;
	} else {
		enable = true;
	}
}

/// ʏ
void MsgBoard::InitScreen(SDL_PixelFormat *format, int width, int height)
{
	// F
	RGB_IN(fg, 0x00, 0xc0, 0x80);
	RGB_IN(bg, 0x00, 0x40, 0x00);

	// EBhETCY
	szWin.cx = width;
	szWin.cy = height;

	CreateSurface(format, width, 128);

	if (enable) {
		// SDL_ttf 
		if (!TTF_WasInit()) {
			if (TTF_Init() != -1) {
				inited = true;
			} else {
				enable = false;
			}
		}
	}
	// bZ[WptHg̐ݒ
	// gݒ
	RECT_IN(msg.re, 0, 0, width, 63);
	// \ʒu
	msg.pt.x = 0; msg.pt.y = 0; info.place = 2;
	// ptHg̐ݒ
	// gݒ
	RECT_IN(info.re, 0, 64, width, 127);
	// \ʒu
	info.pt.x = 0; info.pt.y = 0; info.place = 1;

	if (enable) {
		SetFont();
	}
	if (enable) {
		emu->out_log(LOG_INFO , _T("MsgBoard: OK"));
	} else {
		emu->out_log(LOG_ERROR, _T("MsgBoard: Failed"));
	}
}

/// tHgݒ
bool MsgBoard::SetFont()
{
	if (!inited) return false;

	// bZ[WptHg̐ݒ
	enable = set_sys_font("message", config.msgboard_msg_fontname, config.msgboard_msg_fontsize, &msg.font, msg.font_name);
	if (enable) {
		TTF_SetFontStyle(msg.font, TTF_STYLE_NORMAL);
		config.msgboard_msg_fontname.Set(msg.font_name);
		// ptHg̐ݒ
		enable = set_sys_font("info", config.msgboard_info_fontname, config.msgboard_info_fontsize, &info.font, info.font_name);
		if (enable) {
			TTF_SetFontStyle(info.font, TTF_STYLE_NORMAL);
			config.msgboard_info_fontname.Set(info.font_name);
		}
	}
	return enable;
}

/// o
void MsgBoard::draw(SDL_Surface *screen, msg_data_t &data)
{
	SDL_Rect reDst;

	if (!data.lists.empty()) {
//		list_t::iterator it = data.lists.begin();

		// ʒǔvZ
		if (data.place & 1) {
			reDst.x = szWin.cx + data.pt.x - data.sz.cx;
			reDst.w = data.sz.cx;
		} else {
			reDst.x = data.pt.x;
			reDst.w = data.sz.cx;
		}
		if (data.place & 2) {
			reDst.y = szWin.cy + data.pt.y - data.sz.cy;
			reDst.h = data.sz.cy;
		} else {
			reDst.y = data.pt.y;
			reDst.h = data.sz.cy;
		}

		// CReLXgɃbZ[WRs[
#ifdef USE_BG_TRANSPARENT
		draw_text(screen, data, reDst.x, reDst.y);
#else
		SDL_BlitSurface(sMainSuf, &data.re,
			screen, &reDst);
#endif
	}
}

void MsgBoard::Draw(SDL_Surface *screen)
{
	if (!enable || !visible) return;

	draw(screen, msg);
	draw(screen, info);
}

/// obNobt@ɕ`
void MsgBoard::draw_text(msg_data_t &data)
{
	SDL_Surface *sTextSuf = NULL;

	int cx = 0;
	int cy = 0;
	int len = 0;

	if (enable && !data.lists.empty()) {
		list_t::iterator it = data.lists.begin();

#ifndef USE_BG_TRANSPARENT
		// g`
		SDL_FillRect(sMainSuf, &data.re, SDL_MapRGB(sMainSuf->format, bg.r, bg.g, bg.b));

		// `
		len = (int)strlen(it->cmsg);
		sTextSuf = TTF_RenderUTF8_Shaded(data.font, it->cmsg, fg, bg);
		if (sTextSuf != NULL) {
			SDL_BlitSurface(sTextSuf, NULL, sMainSuf, &data.re);

			// ̘̕gvZ
			TTF_SizeUTF8(data.font, it->cmsg, &cx, &cy);
			data.sz.cx = cx + 4;
			data.sz.cy = cy + 4;

			SDL_FreeSurface(sTextSuf);

		} else {
			// G[
			data.sz.cx = 0;
			data.sz.cy = 0;
		}
#else
		// ̘̕gvZ
		TTF_SizeUTF8(data.font, it->cmsg, &cx, &cy);
		data.sz.cx = cx + 4;
		data.sz.cy = cy + 4;
#endif
	}
}

/// obNobt@ɕ`
void MsgBoard::draw_text(SDL_Surface *suf, msg_data_t &data, int left, int top)
{
	SDL_Surface *sTextSuf = NULL;
	SDL_Rect re;
	int len = 0;

	if (enable && !data.lists.empty()) {
		list_t::iterator it = data.lists.begin();

		// `
		len = (int)strlen(it->cmsg);
		sTextSuf = TTF_RenderUTF8_Blended(data.font, it->cmsg, fg);
		if (sTextSuf != NULL) {
			re.x = data.re.x + left; re.y = top;
			SDL_BlitSurface(sTextSuf, NULL, suf, &re);
			SDL_FreeSurface(sTextSuf);
		}
	}
}

/// bZ[Wݒ
void MsgBoard::Set(msg_data_t &data, const _TCHAR *str, int sec)
{
	item_t itm;

	UTILITY::wcs_to_mbs(itm.cmsg, str, 1024);
	itm.cnt = (60 * sec);

	data.lists.push_front(itm);
	if (data.lists.size() > 10) {
		data.lists.pop_back();
	}

	if (enable) {
		draw_text(data);
	}
}
void MsgBoard::SetMessage(const _TCHAR *str, int sec)
{
	Set(msg, str, sec);
}
void MsgBoard::SetInfo(const _TCHAR *str, int sec)
{
	Set(info, str, sec);
}
void MsgBoard::SetMessageF(const _TCHAR *format, ...)
{
	_TCHAR buf[1024];

	va_list ap;

	va_start(ap, format);
	_vstprintf(buf, format, ap);
	va_end(ap);

	SetMessage(buf);
}
void MsgBoard::SetInfoF(const _TCHAR *format, ...)
{
	_TCHAR buf[1024];

	va_list ap;

	va_start(ap, format);
	_vstprintf(buf, format, ap);
	va_end(ap);

	SetInfo(buf);
}

/// EBhETCYݒ
void MsgBoard::SetSize(int width, int height)
{
	szWin.cx = width;
	szWin.cy = height;
}

/// bZ[W`ʒuݒ
void MsgBoard::SetMessagePos(int cx, int cy, int place)
{
	msg.pt.x = cx;
	msg.pt.y = cy;
	msg.place = place;
}
void MsgBoard::SetInfoPos(int cx, int cy, int place)
{
	info.pt.x = cx;
	info.pt.y = cy;
	info.place = place;
}

/// bZ[W폜
void MsgBoard::Delete(msg_data_t &data, const _TCHAR *str)
{
	bool redraw = false;
	char cmsg[1024];

	if (!data.lists.empty()) {
		UTILITY::wcs_to_mbs(cmsg, str, 1024);
		list_t::iterator it = data.lists.begin();
		list_t::iterator it_next;
		while(it != data.lists.end())	{
			it_next = it;
			it_next++;

			if (strcmp(cmsg, it->cmsg) == 0) {
				data.lists.erase(it);
				redraw = true;
			}
			it = it_next;
		}
	}
	if (redraw) {
		draw_text(data);
	}
}
void MsgBoard::DeleteMessage(const _TCHAR *str)
{
	Delete(msg, str);
}
void MsgBoard::DeleteInfo(const _TCHAR *str)
{
	Delete(info, str);
}

/// JEg_E
void MsgBoard::count_down(msg_data_t &data)
{
	bool redraw = false;
	if (!data.lists.empty()) {
		list_t::iterator it = data.lists.begin();
		list_t::iterator it_next;
		while(it != data.lists.end())	{
			it_next = it;
			it_next++;

			if (it->cnt > 0) it->cnt--;
			if (it->cnt == 0) {
				data.lists.erase(it);
				redraw = true;
			}
			it = it_next;
		}
	}
	if (redraw) {
		draw_text(data);
	}
}
void MsgBoard::CountDown(void)
{
	count_down(msg);
	count_down(info);
}

/// ʕ\pVXetHg̐ݒ
bool MsgBoard::set_sys_font(const char *title, const _TCHAR *name, int pt, TTF_Font **font, char *font_file)
{
	sbuf_t sbuf;
	std::list<sbuf_t> fpath;
	std::list<sbuf_t> fname;
	TTF_Font *f;

#if defined(_WIN32)
	UTILITY::wcs_to_mbs(sbuf.buf, config.font_path, _MAX_PATH);
	fpath.push_back(sbuf);
	UTILITY::wcs_to_mbs(sbuf.buf, emu->resource_path(), _MAX_PATH);
	fpath.push_back(sbuf);
	UTILITY::wcs_to_mbs(sbuf.buf, emu->application_path(), _MAX_PATH);
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s\\fonts\\", getenv("SystemRoot"));
	fpath.push_back(sbuf);

	UTILITY::wcs_to_mbs(sbuf.buf, name, _MAX_PATH);
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "msgothic.ttc");
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "arial.ttf");
	fname.push_back(sbuf);

#elif defined(linux)
	UTILITY::wcs_to_mbs(sbuf.buf, config.font_path, _MAX_PATH);
	fpath.push_back(sbuf);
	UTILITY::wcs_to_mbs(sbuf.buf, emu->resource_path(), _MAX_PATH);
	fpath.push_back(sbuf);
	UTILITY::wcs_to_mbs(sbuf.buf, emu->application_path(), _MAX_PATH);
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s/.fonts/", getenv("HOME"));
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/usr/share/fonts/truetype/");
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/usr/share/fonts/opentype/");
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/usr/share/fonts/opentype/ipafont/");
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/usr/share/fonts/truetype/freefont/");
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/usr/X11R6/lib/X11/fonts/TTF/");
	fpath.push_back(sbuf);

	UTILITY::wcs_to_mbs(sbuf.buf, name, _MAX_PATH);
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "ttf-japanese-gothic.ttf");
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "fonts-japanese-gothic.ttf");
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "FreeSans.ttf");
	fname.push_back(sbuf);

#elif defined(__APPLE__) && defined(__MACH__)
	UTILITY::wcs_to_mbs(sbuf.buf, config.font_path, _MAX_PATH);
	fpath.push_back(sbuf);
	UTILITY::wcs_to_mbs(sbuf.buf, emu->resource_path(), _MAX_PATH);
	fpath.push_back(sbuf);
	UTILITY::wcs_to_mbs(sbuf.buf, emu->application_path(), _MAX_PATH);
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s/Library/Fonts/", getenv("HOME"));
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/Library/Fonts/");
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/System/Library/Fonts/");
	fpath.push_back(sbuf);

	UTILITY::wcs_to_mbs(sbuf.buf, name, _MAX_PATH);
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "Hiragino Sans GB W3.otf");
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "Osaka.ttf");
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "AppleGothic.ttf");
	fname.push_back(sbuf);

#elif defined(__FreeBSD__)
	UTILITY::wcs_to_mbs(sbuf.buf, config.font_path, _MAX_PATH);
	fpath.push_back(sbuf);
	UTILITY::wcs_to_mbs(sbuf.buf, emu->resource_path(), _MAX_PATH);
	fpath.push_back(sbuf);
	UTILITY::wcs_to_mbs(sbuf.buf, emu->application_path(), _MAX_PATH);
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "%s/.fonts/", getenv("HOME"));
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/usr/share/fonts/TTF/");
	fpath.push_back(sbuf);
	sprintf(sbuf.buf, "/usr/local/lib/X11/fonts/TTF/");
	fpath.push_back(sbuf);

	UTILITY::wcs_to_mbs(sbuf.buf, name, _MAX_PATH);
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "ipagp-mona.ttf");
	fname.push_back(sbuf);
	strcpy(sbuf.buf, "luxisr.ttf");
	fname.push_back(sbuf);

#endif
	std::list<sbuf_t>::iterator itn = fname.begin();
	while (itn != fname.end()) {
		std::list<sbuf_t>::iterator itp = fpath.begin();
		while (itp != fpath.end()) {
			sprintf(sbuf.buf, "%s%s", itp->buf, itn->buf);
			f = TTF_OpenFont(sbuf.buf, pt);
			if (f != NULL) {
				if (*font != NULL) {
					TTF_CloseFont(*font);
				}
				*font = f;
				emu->out_logf(LOG_INFO, (const char *)"MsgBoard: Use %s for %s.", sbuf.buf, title);
				if (font_file) {
					strncpy(font_file, itn->buf, 128);
				}
				return true;
			}
			itp++;
		}
		itn++;
	}
	emu->out_logf(LOG_WARN, (const char *)"MsgBoard: Couldn't find fonts for %s.", title);
	return false;
}
