/** @file keyboard_bind.h

	HITACHI MB-S1 Emulator 'EmuB-S1'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.04.08 -

	@brief [ keyboard bind ]
*/

#include "../vm.h"
#include "../../keycode.h"

#ifndef USE_SDL2
/***** SDL1 *****/
#define VK_KEY_TYPE "SDL"
#else
/***** SDL2 *****/
#define VK_KEY_TYPE "SDL2"
#endif

/* s1        -> common  win     linux     mac          */
/* [caps]    -> F6      shift + capslock  capslock     */
/* [graph]   -> F7      muhenkan          rightcommand */
/* [kana]    -> F8                        rightoption  */
/* [break]   -> F9                                     */
/* [num ?]   -> end                                    */
/* [num ,]   -> pagedown                               */
/* [henkan]  ->                           rightcommand */
/* [copy]    -> F12                                    */
/* reset     ->         F11               F10          */
/* (pause)   ->         pause             F15          */
/* (alt)     ->         alt               leftoption   */

static const uint32 scan2key_defmap[KEYBIND_KEYS][KEYBIND_ASSIGN]={
#if defined(__APPLE__) && defined(__MACH__)
//
// macosx
//
//	 space                up                            num?                       left                      down                     right                       ctrl                          shift
//   0x80,                0x81,                         0x82,                      0x83,                     0x84,                    0x85,                       0x86,                         0x87,
	{KEYCODE_SPACE,0x00},{KEYCODE_UP,0x00},            {KEYCODE_END,0x00},        {KEYCODE_LEFT,0x00},      {KEYCODE_DOWN,0x00},     {KEYCODE_RIGHT,0x00},       {KEYCODE_LCTRL,KEYCODE_RCTRL},{KEYCODE_LSHIFT,KEYCODE_RSHIFT},
//	                      caps                          kana                       graph                     esc                      num8                        num9                          num*
//   0x88,                0x89,                         0x8a,                      0x8b,                     0x8c,                    0x8d,                       0x8e,                         0x8f,
	{0x00,0x00},         {KEYCODE_CAPSLOCK,KEYCODE_F6},{KEYCODE_RALT,KEYCODE_F8}, {KEYCODE_LGUI,KEYCODE_F7},{KEYCODE_ESCAPE,0x00},   {KEYCODE_KP_8,0x00},        {KEYCODE_KP_9,0x00},          {KEYCODE_KP_MULTIPLY,0x00},
//	 7                    4                             6                          8                         0                        ^                           =-                            3
//   0x90,                0x91,                         0x92,                      0x93,                     0x94,                    0x95,                       0x96,                         0x97
	{KEYCODE_7,0x00},    {KEYCODE_4,0x00},             {KEYCODE_6,0x00},          {KEYCODE_8,0x00},         {KEYCODE_0,0x00},        {KEYCODE_CARET,0x00},       {KEYCODE_MINUS,0x00},         {KEYCODE_3,0x00},
//	                      5                             1                          2                         9                        num7                        bs                            yen(backquote)
//   0x98,                0x99,                         0x9a,                      0x9b,                     0x9c,                    0x9d,                       0x9e,                         0x9f,
	{0x00,0x00},         {KEYCODE_5,0x00},             {KEYCODE_1,0x00},          {KEYCODE_2,0x00},         {KEYCODE_9,0x00},        {KEYCODE_KP_7,0x00},        {KEYCODE_BACKSPACE,0x00},     {KEYCODE_BACKSLASH,0x00},
//	 U                    R                             Y                          I                         P                        [                           @                             num0
//   0xa0,                0xa1,                         0xa2,                      0xa3,                     0xa4,                    0xa5,                       0xa6,                         0xa7,
	{KEYCODE_U,0x00},    {KEYCODE_R,0x00},             {KEYCODE_Y,0x00},          {KEYCODE_I,0x00},         {KEYCODE_P,0x00},        {KEYCODE_LEFTBRACKET,0x00}, {KEYCODE_AT,0x00},            {KEYCODE_KP_0,0x00},
//	 Q                    T                             W                          E                         O                        num .                       home/cls                      return
//   0xa8,                0xa9,                         0xaa,                      0xab,                     0xac,                    0xad,                       0xae,                         0xaf,
	{KEYCODE_Q,0x00},    {KEYCODE_T,0x00},             {KEYCODE_W,0x00},          {KEYCODE_E,0x00},         {KEYCODE_O,0x00},        {KEYCODE_KP_PERIOD,0x00},    {KEYCODE_HOME,0x00},          {KEYCODE_KP_ENTER,KEYCODE_RETURN},
//   J                    F                             H                          K                         +;                       ]                           *:                            num4
//   0xb0,                0xb1,                         0xb2,                      0xb3,                     0xb4,                    0xb5,                       0xb6,                         0xb7,
	{KEYCODE_J,0x00},    {KEYCODE_F,0x00},             {KEYCODE_H,0x00},          {KEYCODE_K,0x00},         {KEYCODE_SEMICOLON,0x00},{KEYCODE_RIGHTBRACKET,0x00},{KEYCODE_COLON,0x00},         {KEYCODE_KP_4,0x00},
//   A                    G                             S                          D                         L                        num5                        num6                          num-
//   0xb8,                0xb9,                         0xba,                      0xbb,                     0xbc,                    0xbd,                       0xbe,                         0xbf,
	{KEYCODE_A,0x00},    {KEYCODE_G,0x00},             {KEYCODE_S,0x00},          {KEYCODE_D,0x00},         {KEYCODE_L,0x00},        {KEYCODE_KP_5,0x00},        {KEYCODE_KP_6,0x00},          {KEYCODE_KP_MINUS,0x00},
//   M                    V                             N                          <,                        ?/                       num/                        _                             num1
//   0xc0,                0xc1,                         0xc2,                      0xc3,                     0xc4,                    0xc5,                       0xc6,                         0xc7,
	{KEYCODE_M,0x00},    {KEYCODE_V,0x00},             {KEYCODE_N,0x00},          {KEYCODE_COMMA,0x00},     {KEYCODE_SLASH,0x00},    {KEYCODE_KP_DIVIDE,0x00},   {KEYCODE_UNDERSCORE,0x00},    {KEYCODE_KP_1,0x00},
//   Z                    B                             X                          C                         >.                       num2                        num3                          num+
//   0xc8,                0xc9,                         0xca,                      0xcb,                     0xcc,                    0xcd,                       0xce,                         0xcf,
	{KEYCODE_Z,0x00},    {KEYCODE_B,0x00},             {KEYCODE_X,0x00},          {KEYCODE_C,0x00},         {KEYCODE_PERIOD,0x00},   {KEYCODE_KP_2,0x00},        {KEYCODE_KP_3,0x00},          {KEYCODE_KP_PLUS,0x00},
//   F1                   F2                            F3                         F4                        F5
//   0xd0,                0xd1,                         0xd2,                      0xd3,                     0xd4,                    0xd5,                       0xd6,                         0xd7,
	{KEYCODE_F1,0x00},   {KEYCODE_F2,0x00},            {KEYCODE_F3,0x00},         {KEYCODE_F4,0x00},        {KEYCODE_F5,0x00},       {0x00,0x00},                {0x00,0x00},                  {0x00,0x00},
//   0xd8,                0xd9,                         0xda,                      0xdb,                     0xdc,                    0xdd,                       0xde,                         0xdf,
	{0x00,0x00},         {0x00,0x00},                  {0x00,0x00},               {0x00,0x00},              {0x00,0x00},             {0x00,0x00},                {0x00,0x00},                  {0x00,0x00},
//                                                                                                                                    ins                                                       henkan
//   0xe0,                0xe1,                         0xe2,                      0xe3,                     0xe4,                    0xe5,                       0xe6,                         0xe7,
	{0x00,0x00},         {0x00,0x00},                  {0x00,0x00},               {0x00,0x00},              {0x00,0x00},             {KEYCODE_INSERT,KEYCODE_FUNCTION},{0x00,0x00},            {KEYCODE_RGUI,0x00},
//   copy                                                                                                    tab                      num ,                                                     del
//   0xe8,                0xe9,                         0xea,                      0xeb,                     0xec,                    0xed,                       0xee,                         0xef,
	{KEYCODE_F12,0x00},  {0x00,0x00},                  {0x00,0x00},               {0x00,0x00},              {KEYCODE_TAB,0x00},      {KEYCODE_PAGEDOWN,0x00},    {0x00,0x00},                  {KEYCODE_DELETE,0x00},
//   0xf0,                0xf1,                         0xf2,                      0xf3,                     0xf4,                    0xf5,                       0xf6,                         0xf7,
	{0x00,0x00},         {0x00,0x00},                  {0x00,0x00},               {0x00,0x00},              {0x00,0x00},             {0x00,0x00},                {0x00,0x00},                  {0x00,0x00},
//  (pause)              (alt)                                                                                                                                                                  reset
	{KEYCODE_F15,0x00},  {KEYCODE_LALT,0x00},          {0x00,0x00},               {0x00,0x00},              {0x00,0x00},             {0x00,0x00},                {0x00,0x00},                  {KEYCODE_F10,0x00},
//   break                (property)
	{KEYCODE_F9,0x00},   {0x00,0x00}

#else
//
// default map
//
//	 space                up                            num?                         left                          down                     right                       ctrl                          shift
//   0x80,                0x81,                         0x82,                        0x83,                         0x84,                    0x85,                       0x86,                         0x87,
	{KEYCODE_SPACE,0x00},{KEYCODE_UP,0x00},            {KEYCODE_END,0x00},          {KEYCODE_LEFT,0x00},          {KEYCODE_DOWN,0x00},     {KEYCODE_RIGHT,0x00},       {KEYCODE_LCTRL,KEYCODE_RCTRL},{KEYCODE_LSHIFT,KEYCODE_RSHIFT},
//	                      caps                          kana                         graph                         esc                      num8                        num9                          num*
//   0x88,                0x89,                         0x8a,                        0x8b,                         0x8c,                    0x8d,                       0x8e,                         0x8f,
	{0x00,0x00},         {KEYCODE_CAPSLOCK,KEYCODE_F6},{KEYCODE_F8,0x00},           {KEYCODE_MUHENKAN,KEYCODE_F7},{KEYCODE_ESCAPE,0x00},   {KEYCODE_KP_8,0x00},        {KEYCODE_KP_9,0x00},          {KEYCODE_KP_MULTIPLY,0x00},
//	 7                    4                             6                            8                             0                        ^                           =-                            3
//   0x90,                0x91,                         0x92,                        0x93,                         0x94,                    0x95,                       0x96,                         0x97
	{KEYCODE_7,0x00},    {KEYCODE_4,0x00},             {KEYCODE_6,0x00},            {KEYCODE_8,0x00},             {KEYCODE_0,0x00},        {KEYCODE_CARET,0x00},       {KEYCODE_MINUS,0x00},         {KEYCODE_3,0x00},
//	                      5                             1                            2                             9                        num7                        bs                            yen(backquote)
//   0x98,                0x99,                         0x9a,                        0x9b,                         0x9c,                    0x9d,                       0x9e,                         0x9f,
	{0x00,0x00},         {KEYCODE_5,0x00},             {KEYCODE_1,0x00},            {KEYCODE_2,0x00},             {KEYCODE_9,0x00},        {KEYCODE_KP_7,0x00},        {KEYCODE_BACKSPACE,0x00},     {KEYCODE_BACKSLASH,0x00},
//	 U                    R                             Y                            I                             P                        [                           @                             num0
//   0xa0,                0xa1,                         0xa2,                        0xa3,                         0xa4,                    0xa5,                       0xa6,                         0xa7,
	{KEYCODE_U,0x00},    {KEYCODE_R,0x00},             {KEYCODE_Y,0x00},            {KEYCODE_I,0x00},             {KEYCODE_P,0x00},        {KEYCODE_LEFTBRACKET,0x00}, {KEYCODE_AT,0x00},            {KEYCODE_KP_0,0x00},
//	 Q                    T                             W                            E                             O                        num .                       home/cls                      return
//   0xa8,                0xa9,                         0xaa,                        0xab,                         0xac,                    0xad,                       0xae,                         0xaf,
	{KEYCODE_Q,0x00},    {KEYCODE_T,0x00},             {KEYCODE_W,0x00},            {KEYCODE_E,0x00},             {KEYCODE_O,0x00},        {KEYCODE_KP_PERIOD,0x00},   {KEYCODE_HOME,0x00},          {KEYCODE_KP_ENTER,KEYCODE_RETURN},
//   J                    F                             H                            K                             +;                       ]                           *:                            num4
//   0xb0,                0xb1,                         0xb2,                        0xb3,                         0xb4,                    0xb5,                       0xb6,                         0xb7,
	{KEYCODE_J,0x00},    {KEYCODE_F,0x00},             {KEYCODE_H,0x00},            {KEYCODE_K,0x00},             {KEYCODE_SEMICOLON,0x00},{KEYCODE_RIGHTBRACKET,0x00},{KEYCODE_COLON,0x00},         {KEYCODE_KP_4,0x00},
//   A                    G                             S                            D                             L                        num5                        num6                          num-
//   0xb8,                0xb9,                         0xba,                        0xbb,                         0xbc,                    0xbd,                       0xbe,                         0xbf,
	{KEYCODE_A,0x00},    {KEYCODE_G,0x00},             {KEYCODE_S,0x00},            {KEYCODE_D,0x00},             {KEYCODE_L,0x00},        {KEYCODE_KP_5,0x00},        {KEYCODE_KP_6,0x00},          {KEYCODE_KP_MINUS,0x00},
//   M                    V                             N                            <,                            ?/                       num/                        _                             num1
//   0xc0,                0xc1,                         0xc2,                        0xc3,                         0xc4,                    0xc5,                       0xc6,                         0xc7,
	{KEYCODE_M,0x00},    {KEYCODE_V,0x00},             {KEYCODE_N,0x00},            {KEYCODE_COMMA,0x00},         {KEYCODE_SLASH,0x00},    {KEYCODE_KP_DIVIDE,0x00},   {KEYCODE_UNDERSCORE,0x00},    {KEYCODE_KP_1,0x00},
//   Z                    B                             X                            C                             >.                       num2                        num3                          num+
//   0xc8,                0xc9,                         0xca,                        0xcb,                         0xcc,                    0xcd,                       0xce,                         0xcf,
	{KEYCODE_Z,0x00},    {KEYCODE_B,0x00},             {KEYCODE_X,0x00},            {KEYCODE_C,0x00},             {KEYCODE_PERIOD,0x00},   {KEYCODE_KP_2,0x00},        {KEYCODE_KP_3,0x00},          {KEYCODE_KP_PLUS,0x00},
//   F1                   F2                            F3                           F4                            F5
//   0xd0,                0xd1,                         0xd2,                        0xd3,                         0xd4,                    0xd5,                       0xd6,                         0xd7,
	{KEYCODE_F1,0x00},   {KEYCODE_F2,0x00},            {KEYCODE_F3,0x00},           {KEYCODE_F4,0x00},            {KEYCODE_F5,0x00},       {0x00,0x00},                {0x00,0x00},                  {0x00,0x00},
//   0xd8,                0xd9,                         0xda,                        0xdb,                         0xdc,                    0xdd,                       0xde,                         0xdf,
	{0x00,0x00},         {0x00,0x00},                  {0x00,0x00},                 {0x00,0x00},                  {0x00,0x00},             {0x00,0x00},                {0x00,0x00},                  {0x00,0x00},
//                                                                                                                                          ins                                                       henkan
//   0xe0,                0xe1,                         0xe2,                        0xe3,                         0xe4,                    0xe5,                       0xe6,                         0xe7,
	{0x00,0x00},         {0x00,0x00},                  {0x00,0x00},                 {0x00,0x00},                  {0x00,0x00},             {KEYCODE_INSERT,0x00},      {0x00,0x00},                  {KEYCODE_HENKAN,0x00},
//   copy                                                                                                          tab                      num ,                                                     del
//   0xe8,                0xe9,                         0xea,                        0xeb,                         0xec,                    0xed,                       0xee,                         0xef,
	{KEYCODE_F12,0x00},  {0x00,0x00},                  {0x00,0x00},                 {0x00,0x00},                  {KEYCODE_TAB,0x00},      {KEYCODE_PAGEDOWN,0x00},    {0x00,0x00},                  {KEYCODE_DELETE,0x00},
//   0xf0,                0xf1,                         0xf2,                        0xf3,                         0xf4,                    0xf5,                       0xf6,                         0xf7,
	{0x00,0x00},         {0x00,0x00},                  {0x00,0x00},                 {0x00,0x00},                  {0x00,0x00},             {0x00,0x00},                {0x00,0x00},                  {0x00,0x00},
//  (pause)              (alt)                                                                                                                                                                        reset
	{KEYCODE_PAUSE,0x00},{KEYCODE_LALT,KEYCODE_RALT},  {0x00,0x00},                 {0x00,0x00},                  {0x00,0x00},             {0x00,0x00},                {0x00,0x00},                  {KEYCODE_F11,0x00},
//   break               (property)
	{KEYCODE_F9,0x00},   {0x00,0x00}
#endif
};

/// joypad button -> key scancode assign
static const uint32 scan2joy_defmap[KEYBIND_KEYS][KEYBIND_ASSIGN]={
//	 space       up          num?        left        down        right       ctrl        shift
//   0x80,       0x81,       0x82,       0x83,       0x84,       0x85,       0x86,       0x87,
	{0x10,0x00},{0x01,0x00},{0x00,0x00},{0x04,0x00},{0x02,0x00},{0x08,0x00},{0x00,0x00},{0x00,0x00},
//	             caps        kana        graph       esc         num8        num9        num*
//   0x88,       0x89,       0x8a,       0x8b,       0x8c,       0x8d,       0x8e,       0x8f,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x01},{0x00,0x09},{0x00,0x00},
//	 7           4           6           8           0           ^           =-          3
//   0x90,       0x91,       0x92,       0x93,       0x94,       0x95,       0x96,       0x97
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//	             5           1           2           9           num7        bs          yen(backquote)
//   0x98,       0x99,       0x9a,       0x9b,       0x9c,       0x9d,       0x9e,       0x9f,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x05},{0x00,0x00},{0x00,0x00},
//	 U           R           Y           I           P           [           @           num0
//   0xa0,       0xa1,       0xa2,       0xa3,       0xa4,       0xa5,       0xa6,       0xa7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x10},
//	 Q           T           W           E           O           num .       home/cls    return
//   0xa8,       0xa9,       0xaa,       0xab,       0xac,       0xad,       0xae,       0xaf,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   J           F           H           K           +;          ]           *:          num4
//   0xb0,       0xb1,       0xb2,       0xb3,       0xb4,       0xb5,       0xb6,       0xb7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x04},
//   A           G           S           D           L           num5        num6        num-
//   0xb8,       0xb9,       0xba,       0xbb,       0xbc,       0xbd,       0xbe,       0xbf,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x08},{0x00,0x00},
//   M           V           N           <,          ?/          num/        _           num1
//   0xc0,       0xc1,       0xc2,       0xc3,       0xc4,       0xc5,       0xc6,       0xc7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x06},
//   Z           B           X           C           >.          num2        num3        num+
//   0xc8,       0xc9,       0xca,       0xcb,       0xcc,       0xcd,       0xce,       0xcf,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x02},{0x00,0x0a},{0x00,0x00},
//   F1          F2          F3          F4          F5
//   0xd0,       0xd1,       0xd2,       0xd3,       0xd4,       0xd5,       0xd6,       0xd7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   0xd8,       0xd9,       0xda,       0xdb,       0xdc,       0xdd,       0xde,       0xdf,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//                                                               ins                     henkan
//   0xe0,       0xe1,       0xe2,       0xe3,       0xe4,       0xe5,       0xe6,       0xe7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   copy                                            tab         num ,                   del
//   0xe8,       0xe9,       0xea,       0xeb,       0xec,       0xed,       0xee,       0xef,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   0xf0,       0xf1,       0xf2,       0xf3,       0xf4,       0xf5,       0xf6,       0xf7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//  (pause)     (alt)                                                                    reset
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   break      (property)
	{0x00,0x00},{0x00,0x00}
};

/// joypad button -> PIA joystick assign
static const uint32 sjoy2joy_defmap[KEYBIND_JOYS][KEYBIND_ASSIGN] = {
//               up          down                    left        up+left     down+left
//   0x00,       0x01,       0x02,       0x03,       0x04,       0x05,       0x06,       0x07,
	{0x00,0x00},{0x01,0x01},{0x02,0x02},{0x00,0x00},{0x04,0x04},{0x05,0x05},{0x06,0x06},{0x00,0x00},
//   right       up+right    down+right
//   0x08,       0x09,       0x0a,       0x0b,       0x0c,       0x0d,       0x0e,       0x0f,
	{0x08,0x08},{0x09,0x09},{0x0a,0x0a},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//               button
//   0x10,       0x11,       0x12,       0x13,       0x14,       0x15,       0x16,       0x17,
	{0x00,0x00},{0x10,0x10},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
};

static const int sjoy2joy_allow_map[] = {
	1,2,4,5,6,8,9,10,-1
};
static const int sjoy2joy_button_map[] = {
	17,-1
};

/// for keybind dialog box
static const uint16 kb2scan_map[128] = {
//  0    1    2    3    4    5    6    7    8    9    a    b    c    d    e    f
	0xff,0x80,0x7f,0xff,0x78,0x79,0x65,0x6f,0x1e,0x6c,0xff,0xff,0x2e,0x2f,0xff,0xff,
	0x07,0x06,0xff,0xff,0x09,0x0a,0x0b,0x67,0xff,0xff,0xff,0x0c,0x05,0x03,0x01,0x04,
	0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x68,0x6d,
	0x14,0x1a,0x1b,0x17,0x11,0x19,0x12,0x10,0x13,0x1c,0x36,0x34,0x43,0x16,0x4c,0x44,
	0x26,0x38,0x49,0x4b,0x3b,0x2b,0x31,0x39,0x32,0x23,0x30,0x33,0x3c,0x40,0x42,0x2c,
	0x24,0x28,0x21,0x3a,0x29,0x20,0x41,0x2a,0x4a,0x22,0x48,0x25,0x1f,0x35,0x15,0x46,
	0x27,0x47,0x4d,0x4e,0x37,0x3d,0x3e,0x1d,0x0d,0x0e,0x0f,0x4f,0x02,0x3f,0x2d,0x45,
	0x50,0x51,0x52,0x53,0x54,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};

static const uint16 kb2sjoy_map[16] = {
//  0    1    2    3    4    5    6    7    8    9    a    b    c    d    e    f
	0xff,0x01,0x09,0x08,0x0a,0x02,0x06,0x04,0x05,0x11,0xff,0xff,0xff,0xff,0xff,0xff
};
